
/** TP ETI5 - CPE Lyon - 2010-2011 */


#include <v3.hpp>
#include <mesh.hpp>
#include <mesh_io_off.hpp>
#include <opengl_drawer.hpp>
#include <matrix3.hpp>
#include <navigator_tool.hpp>
#include <string_helper.hpp>
#include <glut_helper.hpp>
#include <texture.hpp>

#include <GL/glut.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <vector>
#include <spring_mesh.hpp>





//draw the model
int draw_scene();
//load the model
int load_model();
//compute a random number
double get_rand();



//glut functions
int setup_callback();
static void display_callback();
static void reshape_callback (int width, int height);
static void keyboard_callback(unsigned char key,int x,int y);
static void idle_callback();
static void motion_callback (int x, int y);
static void mouse_callback (int button, int action, int x, int y);



using namespace cpe;

static double current_time = 0;
static double last_time = 0;
static double info_last_time = 0;


//GLOBAL VARIABLES
//************************************//
unsigned int counter=0;
navigator_tool nav;



//maillage d'affichage
cpe::mesh mesh_draw;
std::vector<double> mesh_normal;

//raffraichissement de la MAJ de l'affichage
#define FRAMERATE 20
//tissu
spring_mesh smesh;
//pas de temps
double stepsize=0.1;



//load the model
int load_model()
{
    try
    {
        smesh=spring_mesh(30);

        smesh.get_K_structural() = 1000;
        smesh.get_K_shear()      = 1000;
        smesh.get_K_bending()    = 1000;

        stepsize=0.015;


        opengl_drawer::generate_texture(1,"data/cloth2.ppm");
        glBindTexture(GL_TEXTURE_2D,1);


    }
    catch(std::string e)
    {
        std::cout<<"Exception: "<<e<<std::endl;
    }

    return 0;
}


//Draw the scene
int draw_scene()
{
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(5,5);


    glEnable(GL_LIGHTING);
    glColor3d(0.7,0.7,0.7);
    if(nav.is_texture())
        glEnable(GL_TEXTURE_2D);
    else
        glDisable(GL_TEXTURE_2D);


    //sauvegarde periodique du maillage a afficher
    //(independamment du calcul des forces et de l'integration)
    current_time = glutGet (GLUT_ELAPSED_TIME) / 1000.0;
    info_last_time += current_time - last_time;
    last_time = current_time;
    if (info_last_time >= 1.0 / FRAMERATE)
    {
        mesh_draw=smesh.to_mesh();
        mesh_normal=mesh_draw.normal_vertex();
        smesh.store_vertices();

        info_last_time=0.0;
    }

    //calcul des forces
    smesh.compute_force();
    //integration temporelle
    smesh.apply_force(stepsize);

    //affichage
    glDisable(GL_LIGHTING);
    glColor3d(0.2,0.2,0.2);
    if(nav.is_grid())
        cpe::opengl_drawer::draw(smesh); //affichage des quads en fil-de-fer

    glEnable(GL_LIGHTING);
    glColor3d(0.7,0.7,0.7);
    cpe::opengl_drawer::draw(mesh_draw,mesh_normal); //affichage du maillage

    return 0;
}















int main(int argc, char *argv[])
{
    std::cout<<"**hello**\n"<<std::endl;
    std::cout<<"run "<<argv[0]<<" with "<<argc-1<<" parameters ... \n"<<std::endl;


  // ************* //
  // start OpenGL
  // ************* //

  //window size
  nav.screen_size_x()=800;
  nav.screen_size_y()=800;



  //init opengl & glut
  setup_opengl(argc,argv,nav);
  //setup callback functions
  setup_callback();





  // ************* //
  // RUN
  // ************* //
  print_help_start();

  //do start once
  load_model();

  //glut loop
  glutMainLoop ();


  return 0;
}



// *******************//
// Helper
// *******************//

//just a rand number \in [0,1]
double get_rand()
{
  int precision=50000;
  return double(rand()%precision)/double(precision);
}






//*******************//
//GLUT STUFF
//*******************//


//glut functions
int setup_callback()
{

    //callback
    glutDisplayFunc (display_callback);
    glutReshapeFunc (reshape_callback);
    glutKeyboardFunc(keyboard_callback);
    glutIdleFunc(idle_callback);
    glutMotionFunc (motion_callback);
    glutMouseFunc (mouse_callback);

    return 0;
}



//display
static void display_callback()
{
    display_manager_begin(nav);
    draw_scene();
    display_manager_end(nav,counter);
}


static void reshape_callback (int width, int height) {reshape_manager(width,height,nav);}
static void idle_callback() {glutPostRedisplay ();}
static void keyboard_callback(unsigned char key,int x,int y) {keyboard_manager(key,x,y,nav);}
static void motion_callback (int x, int y) {motion_manager(x,y,nav);}
static void mouse_callback (int button, int action, int x, int y) {mouse_manager(button,action,x,y,nav);}












