/*
**    TP ETI5 CPE Lyon
**    Copyright (C) 2010 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <spring_mesh.hpp>

#include <mesh.hpp>

namespace cpe
{


    void spring_mesh::apply_force(const double& dt)
    {

        if(speed.size()!=3*N*N)
            speed.resize(3*N*N);


        //TODO:
        //speed = (1-damping)*speed + dt*force
        //vertices += dt*speed


    }


    void spring_mesh::compute_force()
    {
        if(forces.size()!=3*N*N)
            forces.resize(3*N*N);

        for(unsigned int k=0,Nk=forces.size();k<Nk;++k)
            forces[k]=0.0;

        //gravity
        for(unsigned int k=0;k<N*N;++k)
        {
            forces[3*k+0]=0;
            forces[3*k+1]=0;
            forces[3*k+2]=-1;
        }


        for(unsigned int kx=0;kx<N;++kx)
        {
            for(unsigned int ky=0;ky<N;++ky)
            {

                v3 x_current = v3(vertices[3*(kx+N*ky)+0],vertices[3*(kx+N*ky)+1],vertices[3*(kx+N*ky)+2]);

                //TODO
                // for neighbors
                // {
                //   x_neigh = vertices[3*(kx+1+N*0...
                //
                //   L0=longueur au repos
                //   L = (x_current-x_neigh).norm();
                //
                //   force += K * (L0-L) * (x_current-x_neigh)/L;
                // }
                //

            }
        }


    }










    spring_mesh::spring_mesh(const unsigned int& _N)
        :N(_N),K_structural(1.0),K_shear(1.0),K_bending(1.0),damping(0.1)
    {
        vertices.resize(3*N*N);
        init();
    }

    void spring_mesh::init()
    {
        std::vector<float> u(N);
        for(unsigned int k=0;k<N;++k)
            u[k]=static_cast<double>(k)/static_cast<double>(N-1);

        for(unsigned int kx=0;kx<N;++kx)
        {
            for(unsigned int ky=0;ky<N;++ky)
            {
                vertices[3*(kx+N*ky)+0]=u[kx];
                vertices[3*(kx+N*ky)+1]=u[ky];
                vertices[3*(kx+N*ky)+2]=0;
            }
        }
    }



    const unsigned int& spring_mesh::size() const
    {
        return N;
    }

    mesh spring_mesh::to_mesh() const
    {
        cpe::mesh m;
        //vertices
        for(unsigned int k=0,Nk=vertices.size();k<Nk;++k)
            m.get_vertex().push_back(vertices[k]);
        //texture
        for(unsigned int kx=0;kx<N;++kx)
        {
            for(unsigned int ky=0;ky<N;++ky)
            {
                m.get_texture().push_back(static_cast<double>(kx)/static_cast<double>(N-1));
                m.get_texture().push_back(static_cast<double>(ky)/static_cast<double>(N-1));
            }
        }
        for(unsigned int kx=0;kx<N-1;++kx)
        {
            for(unsigned int ky=0;ky<N-1;++ky)
            {
                m.add_triangle( (kx+0)+N*(ky+0),(kx+1)+N*(ky+0),(kx+1)+N*(ky+1) );
                m.add_triangle( (kx+0)+N*(ky+0),(kx+1)+N*(ky+1),(kx+0)+N*(ky+1) );
            }
        }

        return m;
    }


    const std::vector<float>& spring_mesh::get_vertex() const {return vertices;}
    std::vector<float>& spring_mesh::get_vertex() {return vertices;}

    const std::vector<float>& spring_mesh::get_vertex_storage() const {return vertices_store;}
    std::vector<float>& spring_mesh::get_vertex_storage() {return vertices_store;}

    float& spring_mesh::get_K_structural() {return K_structural;}
    const float& spring_mesh::get_K_structural() const {return K_structural;}
    float& spring_mesh::get_K_shear() {return K_shear;}
    const float& spring_mesh::get_K_shear() const {return K_shear;}
    float& spring_mesh::get_K_bending() {return K_bending;}
    const float& spring_mesh::get_K_bending() const {return K_bending;}
    float& spring_mesh::get_damping() {return damping;}
    const float& spring_mesh::get_damping() const {return damping;}

    void spring_mesh::store_vertices() {vertices_store=vertices;}
}
